@@ -92,7 +92,11 @@ function getMenuPopupHistory(options) {
  * @param {array}  options['urls']                    (optional)
  * @param {string} options['url'][]['label']
  * @param {string} options['url'][]['url']
- * @param {string} options['filter_application']      (optional) Application name for filter by application.
+ * @param {array}  options['tags']                    (optional)
+ * @param {string} options['tags'][]['tag']
+ * @param {string} options['tags'][]['value']
+ * @param {number} options['tags'][]['operator']
+ * @param {number} options['evaltype']                (optional)
  * @param {bool}   options['allowed_ui_inventory']    Whether user has access to inventory hosts page.
  * @param {bool}   options['allowed_ui_latest_data']  Whether user has access to latest data page.
  * @param {bool}   options['allowed_ui_problems']     Whether user has access to problems page.
@@ -134,8 +138,9 @@ function getMenuPopupHost(options, trigger_elmnt) {
 		// latest data link
 		var url = new Curl('zabbix.php', false);
 		url.setArgument('action', 'latest.view');
-		if (typeof options.filter_application !== 'undefined') {
-			url.setArgument('filter_application', options.filter_application);
+		if (typeof options.tags !== 'undefined') {
+			url.setArgument('filter_tags', options.tags);
+			url.setArgument('filter_evaltype', options.evaltype);
 		}
 		url.setArgument('filter_hostids[]', options.hostid);
 		url.setArgument('filter_set', '1');
@@ -155,8 +160,9 @@ function getMenuPopupHost(options, trigger_elmnt) {
 			if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
 				url.setArgument('show_suppressed', '1');
 			}
-			if (typeof options.filter_application !== 'undefined') {
-				url.setArgument('application', options.filter_application);
+			if (typeof options.tags !== 'undefined') {
+				url.setArgument('tags', options.tags);
+				url.setArgument('evaltype', options.evaltype);
 			}
 
 			problems.url = url.getUrl();
@@ -256,7 +262,7 @@ function getMenuPopupHost(options, trigger_elmnt) {
 	if (typeof options.scripts !== 'undefined') {
 		sections.push({
 			label: t('Scripts'),
-			items: getMenuPopupScriptData(options.scripts, options.hostid, trigger_elmnt)
+			items: getMenuPopupScriptData(options.scripts, trigger_elmnt, options.hostid)
 		});
 	}
 
@@ -267,41 +273,46 @@ function getMenuPopupHost(options, trigger_elmnt) {
  * Get menu popup submap map element section data.
  *
  * @param {array}  options['sysmapid']
- * @param {int}    options['severity_min']     (optional)
- * @param {int}    options['widget_uniqueid']  (optional)
- * @param {array}  options['urls']             (optional)
+ * @param {int}    options['severity_min']    (optional)
+ * @param {int}    options['is_widget']       (optional)
+ * @param {array}  options['urls']            (optional)
  * @param {string} options['url'][]['label']
  * @param {string} options['url'][]['url']
  *
  * @return array
  */
 function getMenuPopupMapElementSubmap(options) {
-	var sections = [],
-		submap_url;
+	const sections = [];
+	const item = {label: t('Submap')};
 
-	if (typeof options.widget_uniqueid !== 'undefined') {
-		submap_url = new Curl('javascript: navigateToSubmap(' + options.sysmapid +
-			', "' + options.widget_uniqueid + '");', false);
+	if (options.unique_id !== undefined) {
+		item.clickCallback = ()=> {
+			ZABBIX.Dashboard.getDashboardPages().forEach((page) => {
+				const widget = page.getWidget(options.unique_id);
+
+				if (widget !== null) {
+					widget.navigateToSubmap(options.sysmapid);
+				}
+			});
+		};
 	}
 	else {
 		if (!options.allowed_ui_maps) {
 			return [];
 		}
 
-		submap_url = new Curl('zabbix.php', false);
+		const submap_url = new Curl('zabbix.php', false);
 		submap_url.setArgument('action', 'map.view');
 		submap_url.setArgument('sysmapid', options.sysmapid);
 		if (typeof options.severity_min !== 'undefined') {
 			submap_url.setArgument('severity_min', options.severity_min);
 		}
+		item.url = submap_url.getUrl();
 	}
 
 	sections.push({
 		label: t('Go to'),
-		items: [{
-			label: t('Submap'),
-			url: submap_url.getUrl()
-		}]
+		items: [item]
 	});
 
 	// urls
@@ -324,7 +335,11 @@ function getMenuPopupMapElementSubmap(options) {
  * @param {array}  options['urls']               (optional)
  * @param {string} options['url'][]['label']
  * @param {string} options['url'][]['url']
- * @param {string} options['filter_application'] (optional) Application name for filter by application.
+ * @param {array}  options['tags']               (optional)
+ * @param {string} options['tags'][]['tag']
+ * @param {string} options['tags'][]['value']
+ * @param {number} options['tags'][]['operator']
+ * @param {number} options['evaltype']           (optional)
  *
  * @return array
  */
@@ -345,8 +360,9 @@ function getMenuPopupMapElementGroup(options) {
 	if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
 		problems_url.setArgument('show_suppressed', '1');
 	}
-	if (typeof options.filter_application !== 'undefined') {
-		problems_url.setArgument('application', options.filter_application);
+	if (typeof options.tags !== 'undefined') {
+		problems_url.setArgument('tags', options.tags);
+		problems_url.setArgument('evaltype', options.evaltype);
 	}
 
 	sections.push({
@@ -439,289 +455,107 @@ function getMenuPopupMapElementImage(options) {
 }
 
 /**
- * Get menu popup refresh section data.
- *
- * @param {integer}  options['widgetid']	 Widget ID.
- * @param {string}   options['widgetName']   Widget name.
- * @param {string}   options['currentRate']  Current rate value.
- * @param {bool}     options['multiplier']   Multiplier or time mode.
- * @param {array}    options['params']       Url parameters (optional).
- * @param {callback} options['callback']     Callback function on success (optional).
- * @param {object}   trigger_elmnt           UI element which triggered opening of overlay dialogue.
- *
- * @return array
- */
-function getMenuPopupRefresh(options, trigger_elmnt) {
-	var items = [],
-		params = (typeof options.params === 'undefined' || options.params.length == 0) ? {} : options.params,
-		intervals = options.multiplier
-			? {
-				'x0.25': 'x0.25',
-				'x0.5': 'x0.5',
-				'x1': 'x1',
-				'x1.5': 'x1.5',
-				'x2': 'x2',
-				'x3': 'x3',
-				'x4': 'x4',
-				'x5': 'x5'
-			}
-			: {
-				0: t('No refresh'),
-				10: t('10 seconds'),
-				30: t('30 seconds'),
-				60: t('1 minute'),
-				120: t('2 minutes'),
-				600: t('10 minutes'),
-				900: t('15 minutes')
-			};
-
-	jQuery.each(intervals, function(value, label) {
-		var item = {
-			label: label,
-			data: {
-				value: value
-			},
-			clickCallback: function() {
-				var $obj = jQuery(this),
-					currentRate = $obj.data('value');
-
-				// it is a quick solution for slide refresh multiplier, should be replaced with slide.refresh or similar
-				if (options.multiplier) {
-					sendAjaxData('slides.php', {
-						data: jQuery.extend({}, params, {
-							widgetName: options.widgetName,
-							widgetRefreshRate: currentRate
-						}),
-						dataType: 'script',
-						success: function() {
-							// Set new refresh rate as current in slideshow controls.
-							trigger_elmnt.data('menu-popup').data.currentRate = currentRate;
-						}
-					});
-
-					jQuery('a', $obj.closest('.menu-popup')).each(function() {
-						var link = jQuery(this);
-
-						if (link.data('value') == currentRate) {
-							link
-								.addClass('selected')
-								.attr('aria-label', sprintf(t('S_SELECTED_SR'), link.data('aria-label')));
-						}
-						else {
-							link
-								.removeClass('selected')
-								.attr('aria-label', link.data('aria-label'));
-						}
-					});
-
-					$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
-				}
-				else {
-					var url = new Curl('zabbix.php');
-					url.setArgument('action', 'dashboard.widget.rfrate');
-
-					jQuery.ajax({
-						url: url.getUrl(),
-						method: 'POST',
-						dataType: 'json',
-						data: {
-							'widgetid': options.widgetid,
-							'rf_rate': currentRate
-						}
-					})
-						.then(function() {
-							jQuery('a', $obj.closest('.menu-popup')).each(function() {
-								var link = jQuery(this);
-
-								if (link.data('value') == currentRate) {
-									link
-										.addClass('selected')
-										.attr('aria-label', sprintf(t('S_SELECTED_SR'), link.data('aria-label')));
-								}
-								else {
-									link
-										.removeClass('selected')
-										.attr('aria-label', link.data('aria-label'));
-								}
-							});
-
-							// Set new refresh rate as current in widget controls.
-							trigger_elmnt.data('menu-popup').data.currentRate = currentRate;
-
-							$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
-
-							jQuery('.dashbrd-grid-container')
-								.dashboardGrid('setWidgetRefreshRate', options.widgetid, parseInt(currentRate));
-						})
-						.fail(function() {
-							$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
-							// TODO: gentle message about failed saving of widget refresh rate
-						});
-				}
-			}
-		};
-
-		if (value == options.currentRate) {
-			item.selected = true;
-		}
-
-		items[items.length] = item;
-	});
-
-	return [{
-		label: options.multiplier ? t('Refresh interval multiplier') : t('Refresh interval'),
-		items: items
-	}];
-}
-
-/**
- * Get menu popup widget actions data.
- *
- * @param {string}   options['currentRate']      Current rate value.
- * @param {bool}     options['multiplier']       Multiplier or time mode.
- * @param {callback} options['callback']         Callback function on success (optional).
- * @param {string}   options['widget_uniqueid']  Widget instance unique id.
- * @param {object}   trigger_elmnt               UI element which triggered opening of overlay dialogue.
- *
- * @return array
- */
-function getMenuPopupWidgetActions(options, trigger_elmnt) {
-	var $dashboard = jQuery('.dashbrd-grid-container'),
-		dashboard_data = $dashboard.dashboardGrid('getDashboardData'),
-		editMode = $dashboard.dashboardGrid('isEditMode'),
-		widget = $dashboard.dashboardGrid('getWidgetsBy', 'uniqueid', options.widget_uniqueid).pop(),
-		widgetid = widget.widgetid,
-		loading = (!widget['ready'] || widget['content_body'].find('.is-loading').length > 0),
-		widget_actions = [],
-		menu;
-
-	options.widgetid = widgetid;
-	menu = editMode ? [] : getMenuPopupRefresh(options, trigger_elmnt);
-
-	// Do not show "Copy" action for host dashboards.
-	if ($dashboard.data('dashboardGrid')['options']['allowed_edit']
-			&& (dashboard_data.templateid === null || dashboard_data.dynamic_hostid === null)) {
-		widget_actions.push({
-			label: t('S_COPY'),
-			clickCallback: function() {
-				jQuery('.dashbrd-grid-container').dashboardGrid('copyWidget', widget);
-				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
-			}
-		});
-	}
-
-	if (editMode) {
-		widget_actions.push({
-			label: t('S_PASTE'),
-			disabled: ($dashboard.dashboardGrid('getCopiedWidget') === null),
-			clickCallback: function() {
-				$dashboard.dashboardGrid('pasteWidget', widget, widget.pos);
-				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
-			}
-		});
-
-		widget_actions.push({
-			label: t('Delete'),
-			clickCallback: function() {
-				jQuery('.dashbrd-grid-container').dashboardGrid('deleteWidget', widget);
-				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
-			}
-		});
-	}
-
-	if ('download' in options && !editMode) {
-		widget_actions.push({
-			label: t('Download image'),
-			disabled: loading || !options.download,
-			clickCallback: function() {
-				var svg = widget['content_body'].find('svg').first();
-
-				if (svg.length) {
-					downloadSvgImage(svg, 'graph.png');
-				}
-				else {
-					downloadPngImage(widget['content_body'].find('img').first(), 'graph.png');
-				}
-
-				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
-			},
-			refreshCallback: function(widget) {
-				if (widget['widgetid'] == widgetid && options.download) {
-					this.disabled = !widget['ready'];
-				}
-			}
-		});
-	}
-
-	if (widget_actions.length) {
-		menu.unshift({
-			label: t('Actions'),
-			items: widget_actions
-		});
-	}
-
-	return menu;
-}
-
-/**
- * Get menu popup trigger section data.
+ * Get menu popup dashboard actions data.
  *
  * @param {string} options['dashboardid']
  * @param {bool}   options['editable']
- * @param {object} trigger_elmnt           UI element which triggered opening of overlay dialogue.
+ * @param {bool}   options['has_related_reports']
+ * @param {bool}   options['can_edit_dashboards']
+ * @param {bool}   options['can_view_reports']
+ * @param {bool}   options['can_create_reports']
+ * @param {object} trigger_elmnt                   UI element which triggered opening of overlay dialogue.
  *
  * @return array
  */
 function getMenuPopupDashboard(options, trigger_elmnt) {
-	var	url_create = new Curl('zabbix.php', false),
-		url_clone = new Curl('zabbix.php', false),
-		url_delete = new Curl('zabbix.php', false);
+	const sections = [];
+	const popup_options = {
+		dashboardid: options.dashboardid
+	};
 
-	url_create.setArgument('action', 'dashboard.view');
-	url_create.setArgument('new', '1');
+	// Dashboard actions.
+	if (options.can_edit_dashboards) {
+		const url_create = new Curl('zabbix.php', false);
+		url_create.setArgument('action', 'dashboard.view');
+		url_create.setArgument('new', '1');
 
-	url_clone.setArgument('action', 'dashboard.view');
-	url_clone.setArgument('source_dashboardid', options.dashboardid);
+		const url_clone = new Curl('zabbix.php', false);
+		url_clone.setArgument('action', 'dashboard.view');
+		url_clone.setArgument('source_dashboardid', options.dashboardid);
 
-	url_delete.setArgument('action', 'dashboard.delete');
-	url_delete.setArgument('dashboardids', [options.dashboardid]);
+		const url_delete = new Curl('zabbix.php', false);
+		url_delete.setArgument('action', 'dashboard.delete');
+		url_delete.setArgument('dashboardids', [options.dashboardid]);
 
-	return [{
-		label: t('Actions'),
-		items: [
+		sections.push({
+			label: t('Actions'),
+			items: [
+				{
+					label: t('Sharing'),
+					clickCallback: function () {
+						jQuery(this).closest('.menu-popup').menuPopup('close', null);
+
+						PopUp('dashboard.share.edit', popup_options, 'dashboard_share', trigger_elmnt);
+					},
+					disabled: !options.editable
+				},
+				{
+					label: t('Create new'),
+					url: url_create.getUrl()
+				},
+				{
+					label: t('Clone'),
+					url: url_clone.getUrl()
+				},
+				{
+					label: t('Delete'),
+					clickCallback: function () {
+						jQuery(this).closest('.menu-popup').menuPopup('close', null);
+
+						if (!confirm(t('Delete dashboard?'))) {
+							return false;
+						}
+
+						redirect(url_delete.getUrl(), 'post', 'sid', true, true);
+					},
+					disabled: !options.editable
+				}
+			]
+		});
+	}
+
+	// Report actions.
+	if (options.can_view_reports) {
+		const report_actions = [
 			{
-				label: t('Sharing'),
+				label: t('View related reports'),
 				clickCallback: function () {
 					jQuery(this).closest('.menu-popup').menuPopup('close', null);
 
-					var popup_options = {'dashboardid': options.dashboardid};
-					PopUp('dashboard.share.edit', popup_options, 'dashboard_share', trigger_elmnt);
+					PopUp('popup.scheduledreport.list', popup_options, null, trigger_elmnt);
 				},
-				disabled: !options.editable
-			},
-			{
-				label: t('Create new'),
-				url: url_create.getUrl()
-			},
-			{
-				label: t('Clone'),
-				url: url_clone.getUrl()
-			},
-			{
-				label: t('Delete'),
-				clickCallback: function () {
-					jQuery(this).closest('.menu-popup').menuPopup('close', null);
-
-					if (!confirm(t('Delete dashboard?'))) {
-						return false;
-					}
-
-					redirect(url_delete.getUrl(), 'post', 'sid', true, true);
-				},
-				disabled: !options.editable
+				disabled: !options.has_related_reports
 			}
-		]
-	}];
+		];
+
+		if (options.can_create_reports) {
+			report_actions.unshift({
+				label: t('Create new report'),
+				clickCallback: function () {
+					jQuery(this).closest('.menu-popup').menuPopup('close', null);
+
+					PopUp('popup.scheduledreport.edit', popup_options, null, trigger_elmnt);
+				}
+			});
+		}
+
+		sections.push({
+			label: options.can_edit_dashboards ? null : t('Actions'),
+			items: report_actions
+		})
+	}
+
+	return sections;
 }
 
 /**
@@ -783,6 +617,7 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
 
 		url.setArgument('form', 'update');
 		url.setArgument('triggerid', options.triggerid);
+		url.setArgument('context', 'host');
 
 		items[items.length] = {
 			label: t('Configuration'),
@@ -826,6 +661,14 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
 		};
 	}
 
+	// scripts
+	if (typeof options.scripts !== 'undefined') {
+		sections.push({
+			label: t('Scripts'),
+			items: getMenuPopupScriptData(options.scripts, trigger_elmnt, null, options.eventid)
+		});
+	}
+
 	return sections;
 }
 
@@ -839,6 +682,7 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
  * @param array  options['triggers']                  (optional)
  * @param string options['triggers'][n]['triggerid']
  * @param string options['triggers'][n]['name']
+ * @param string options['context']                   Additional parameter in URL to identify main section.
  * @param {object} trigger_elmnt                      UI element that was clicked to open overlay dialogue.
  *
  * @return array
@@ -891,11 +735,11 @@ function getMenuPopupItem(options, trigger_elmnt) {
 	}
 
 	var url = new Curl('items.php', false);
-
 	url.setArgument('form', 'create');
 	url.setArgument('hostid', options.hostid);
 	url.setArgument('type', 18);	// ITEM_TYPE_DEPENDENT
 	url.setArgument('master_itemid', options.itemid);
+	url.setArgument('context', options.context);
 
 	items.push({
 		label: t('Create dependent item'),
@@ -903,11 +747,12 @@ function getMenuPopupItem(options, trigger_elmnt) {
 		disabled: !options.create_dependent_item
 	});
 
-	url = new Curl('host_discovery.php');
+	url = new Curl('host_discovery.php', false);
 	url.setArgument('form', 'create');
 	url.setArgument('hostid', options.hostid);
 	url.setArgument('type', 18);	// ITEM_TYPE_DEPENDENT
 	url.setArgument('master_itemid', options.itemid);
+	url.setArgument('context', options.context);
 
 	items.push({
 		label: t('Create dependent discovery rule'),
@@ -927,6 +772,7 @@ function getMenuPopupItem(options, trigger_elmnt) {
  * @param array options['name']
  * @param array options['itemid']
  * @param array options['parent_discoveryid']
+ * @param string options['context']                   Additional parameter in URL to identify main section.
  *
  * @return array
  */
@@ -937,6 +783,7 @@ function getMenuPopupItemPrototype(options) {
 	url.setArgument('parent_discoveryid', options.parent_discoveryid);
 	url.setArgument('type', 18);	// ITEM_TYPE_DEPENDENT
 	url.setArgument('master_itemid', options.itemid);
+	url.setArgument('context', options.context);
 
 	return [{
 		label: options.name,
@@ -1091,13 +938,14 @@ function getMenuPopupTriggerMacro(options) {
 /**
  * Build script menu tree.
  *
- * @param array scripts           Scripts names.
- * @param array hostId            Host ID.
+ * @param array scripts           Script names amd nenu paths.
  * @param {object} trigger_elmnt  UI element which triggered opening of overlay dialogue.
+ * @param array hostid            Host ID.
+ * @param array eventid           Event ID.
  *
  * @returns array
  */
-function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
+function getMenuPopupScriptData(scripts, trigger_elmnt, hostid, eventid) {
 	var tree = {};
 
 	var appendTreeItem = function(tree, name, items, params) {
@@ -1105,13 +953,17 @@ function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
 			var item = items.shift();
 
 			if (typeof tree[item] === 'undefined') {
-				tree[item] = {items: {}};
+				tree[item] = {
+					name: item,
+					items: {}
+				};
 			}
 
 			appendTreeItem(tree[item].items, name, items, params);
 		}
 		else {
-			tree[name] = {
+			tree['/' + name] = {
+				name: name,
 				params: params,
 				items: {}
 			};
@@ -1123,33 +975,37 @@ function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
 		var script = scripts[key];
 
 		if (typeof script.scriptid !== 'undefined') {
-			var items = splitPath(script.name),
-				name = (items.length > 0) ? items.pop() : script.name;
+			var items = (script.menu_path.length > 0) ? splitPath(script.menu_path) : [];
 
-			appendTreeItem(tree, name, items, {
-				hostId: hostId,
-				scriptId: script.scriptid,
-				confirmation: script.confirmation
+			appendTreeItem(tree, script.name, items, {
+				scriptid: script.scriptid,
+				confirmation: script.confirmation,
+				hostid: hostid,
+				eventid: eventid
 			});
 		}
 	}
 
-	// build menu items from tree
+	// Build menu items from tree.
 	var getMenuPopupScriptItems = function(tree, trigger_elm) {
 		var items = [];
 
 		if (objectSize(tree) > 0) {
-			jQuery.each(tree, function(name, data) {
-				var item = {label: name};
+			jQuery.each(tree, function(key, data) {
+				var item = {label: data.name};
 
 				if (typeof data.items !== 'undefined' && objectSize(data.items) > 0) {
 					item.items = getMenuPopupScriptItems(data.items, trigger_elm);
 				}
 
-				if (typeof data.params !== 'undefined' && typeof data.params.scriptId !== 'undefined') {
+				if (typeof data.params !== 'undefined' && typeof data.params.scriptid !== 'undefined') {
 					item.clickCallback = function(e) {
-						jQuery(this).closest('.menu-popup-top').menuPopup('close', trigger_elm, false);
-						executeScript(data.params.hostId, data.params.scriptId, data.params.confirmation, trigger_elm);
+						jQuery(this)
+							.closest('.menu-popup-top')
+							.menuPopup('close', trigger_elm, false);
+						executeScript(data.params.scriptid, data.params.confirmation, trigger_elm, data.params.hostid,
+							data.params.eventid
+						);
 						cancelEvent(e);
 					};
 				}
@@ -1164,37 +1020,6 @@ function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
 	return getMenuPopupScriptItems(tree, trigger_elmnt);
 }
 
-/**
- * Create menu for dashboard widget area selector.
- *
- * @param {object} area_selected  Area in which new widget will be created.
- *
- * @returns {array}
- */
-function getDashboardWidgetActionMenu(area_selected) {
-	return [{
-		items: [{
-			label: t('Add widget'),
-			clickCallback: function() {
-				jQuery('.dashbrd-grid-container').dashboardGrid('addNewWidget', null, area_selected);
-			}
-		}, {
-			label: t('Paste widget'),
-			clickCallback: function() {
-				var widget_dims = {
-					x: area_selected.x,
-					y: area_selected.y,
-					width: area_selected.width,
-					height: area_selected.height
-				};
-
-				jQuery('.dashbrd-grid-container').dashboardGrid('pasteWidget', null, widget_dims);
-				jQuery('.dashbrd-grid-new-widget-placeholder').hide();
-			}
-		}]
-	}];
-}
-
 jQuery(function($) {
 
 	/**
@@ -1280,9 +1105,21 @@ jQuery(function($) {
 					 * Please note that click event is also triggered by hitting spacebar on the keyboard,
 					 * in which case the number of mouse clicks (stored in event.originalEvent.detail) will be zero.
 					 */
-					of: (event.type === 'click' && event.originalEvent.detail) ? event : event.target,
+					of: (['click', 'mouseup', 'mousedown'].includes(event.type) && event.originalEvent.detail)
+						? event
+						: event.target,
 					my: 'left top',
-					at: 'left bottom'
+					at: 'left bottom',
+					using: (pos, data) => {
+						let max_left = data.horizontal === 'left'
+							? document.querySelector('.wrapper').clientWidth
+							: document.querySelector('.wrapper').clientWidth - data.element.width;
+
+						pos.left = Math.max(0, Math.min(max_left, pos.left));
+
+						data.element.element[0].style.top = `${pos.top}px`;
+						data.element.element[0].style.left = `${pos.left}px`;
+					}
 				}
 			}, defaultOptions, options || {});
 
@@ -1310,6 +1147,7 @@ jQuery(function($) {
 
 			$menu_popup.data('menu_popup', options);
 
+			$('.wrapper').append($('<div>', {class: 'menu-popup-overlay'}));
 			$('.wrapper').append($menu_popup);
 
 			// Position the menu (before hiding).
@@ -1355,6 +1193,7 @@ jQuery(function($) {
 					$(overlay['element']).attr({'aria-expanded': 'false'});
 				}
 
+				menu_popup.prev().remove();
 				menu_popup.remove();
 
 				// Call menu close callback function.
@@ -1598,6 +1437,10 @@ jQuery(function($) {
 				if ('target' in options) {
 					link.attr('target', options.target);
 				}
+
+				if ('rel' in options) {
+					link.attr('rel', options.rel);
+				}
 			}
 
 			if (typeof options.clickCallback !== 'undefined') {
